#!/bin/bash
#
# (c) 2007 Swisslinux.org fbianco(a)swisslinux(d)org,
# GPL V.3 or later
#
# Some part are based on a script by Leigh Purdie
# https://help.ubuntu.com/community/InstallCDCustomization/Scripts
#

# Some constants
BASEDIR="/home/fbianco/swisslinux/swissbuntu"					# The work directory on the local machine, whitout ending /
SEEDFILENAME="swissbuntu.seed"			# Seed file
SEEDFILE="$BASEDIR/$SEEDFILENAME"
ISOLINUXCFG="$BASEDIR/isolinux.cfg"		# isolinux config file
AUTOPARTEDUDEB="$BASEDIR/autoparted.udeb"	# Autoparted package for remplacing partman
CDIMAGE="$BASEDIR/ubuntu.iso"			# Local Ubuntu ISO image
CDSOURCEDIR="$BASEDIR/cdsource"			# Where the ubuntu iso image will be mounted
FINALDIR="$BASEDIR/SwissbuntuFiles"		# where the ubuntu image will be copied
VOLUMEID="Swissbuntu OEM Install CD"		# Volume ID for mkisofs
CDNAME="Swissbuntu.iso"				# CD name for mkisofs
ISODOWNLOADURL="http://mirror.switch.ch/ftp/mirror/ubuntu-cdimage/7.10/ubuntu-7.10-alternate-i386.iso" # URL of the ubuntu alternate image
GPGKEY="D2681AF6"				# The GPG key used for signig the packages in the iso images

usage()
{
    echo "Usage: $0 [OPTION]"
    echo "
 -f  --full		Copy, patch and build a new iso-image
 -r  --rebuild-only	Rebuild only the iso-image
 -h, --help		Show this usage message.
"
}

failure()
{
	echo "$@"
	exit 1
}

sign()
{
	#
	# Sign the Swissbuntu iso image
	#
	sh swissbuntu_sign.sh $GPGKEY $BASEDIR
	# FIXME add parameter for the key
}

updateMd5sum()
{
	#
	# Update the md5sum of all the packages
	#
	echo	
	echo -n "Do you want to update all the md5 checksums (not needed for testing purpose) ? <y/n>"
	read -p ""
	
	if [ "$REPLY" == "y" ]; then
		echo -n "Updating md5 checksums (go and take a coffee)... "
		cd $FINALDIR
		chmod +w md5sum.txt
		rm -f md5sum.txt
		md5sum `find -follow -type f` > md5sum.txt
		cd ..
		echo "OK"
	fi
}

build()
{
	#
	# Build an Swissbuntu iso image based on the file in $FINALDIR
	#

	if [ ! -d $BASEDIR ]; then 
		failure "ERROR: No directory $BASEDIR found"
# 		exit
	fi
	if [ ! -d $FINALDIR ]; then
		failure "ERROR: No directory $FINALDIR found"
# 		exit
	fi

	if [ -e $BASEDIR/$CDNAME ]; then
		echo "There is already an iso image $CDNAME"
		echo	
		echo -n "Do you want to delete it ? <y/n>"
		read -p ""
		
		if [ "$REPLY" == "y" ]; then
			rm $BASEDIR/$CDNAME
		else
			failure "Process aborted by user"
		fi
	
	fi

	updateMd5sum
	sign

	echo "Creating and ISO image..."
	mkisofs -r -V "$VOLUMEID"			\
		-cache-inodes				\
		-J -l -hide-rr-moved -R			\
		-b isolinux/isolinux.bin		\
		-c isolinux/boot.cat -no-emul-boot	\
		-boot-load-size 4 -boot-info-table	\
		-o $BASEDIR/$CDNAME $FINALDIR
	
	echo
	echo "The new CD image is available in $BASEDIR/$CDNAME"
}

full()
{
	echo "Create an OEM iso image of Ubuntu alternate CD"
	
	# Create a few directories.
	if [ ! -d $BASEDIR ]; then mkdir -p $BASEDIR; fi
	if [ ! -d $FINALDIR ]; then mkdir -p $FINALDIR; fi
	if [ ! -d $CDSOURCEDIR ]; then mkdir -p $CDSOURCEDIR; fi
	
	if [ ! -f $CDIMAGE ]; then
		echo "ERROR: Cannot find your ubuntu (alternate) image."
		echo	
		echo -n "Do you want to download it now (700MB) ? <y/n>"
		read -p ""
	
		if [ "$REPLY" == "y" ]; then
			wget $ISODOWNLOADURL $CDIMAGE
		else
			failure "Create a symlink $CDIMAGE and restart the script"
		fi
	fi
	
	if [ ! -f $SEEDFILE ]; then
		failure "ERROR: Cannot find the seed file."
	fi
	
	if [ ! -f $ISOLINUXCFG ]; then
		failure "ERROR: Cannot find your isolinux config file."
	fi
	
	if [ ! -f $CDSOURCEDIR/md5sum.txt ]; then
	
		echo "Mounting Ubuntu iso..."
		mount | grep $CDSOURCEDIR
		if [ $? -eq 0 ]; then
			umount $CDSOURCEDIR
		fi
	
		echo "Root access is needed to mount the iso image"
		sudo mount -o loop $CDIMAGE $CDSOURCEDIR
		if [ ! -f $CDSOURCEDIR/md5sum.txt ]; then
# 			echo "Mount did not succeed. Exiting."
# 			echo "Did you activate the loop module ?"
			failure "Mount did not succeed. Exiting.\n Did you activate the loop module ? (modprobe loop)"
		fi
		echo "Mounting OK"
	fi
	
	echo "Copying the ubuntu image..."
	# FIXME (?) Use rsync instead of cp
	# rsync -a -H $CDSOURCEDIR/ $FINALDIR/ #-a = archive, -H = preserve link
	cp -a -v -d $CDSOURCEDIR/* $FINALDIR/ # -a = archive, -d = no dereference, preserve link
	
	echo "Root access needed to unmount the iso image"
	sudo umount $CDSOURCEDIR
	rmdir $CDSOURCEDIR
	
	# add write flag to all the files, and do some cleaning job
	chmod -R +w $FINALDIR # FIXME : not really needed, if we know exactly the files we want to modifies

	# Needed for cdrom-detect, else we get an error
	mkdir -p $FINALDIR/.disk
	echo "Swissbuntu" > $FINALDIR/.disk/info	

	# Add the preseed file
	rm -f $FINALDIR/preseed/*
	cp $SEEDFILE $FINALDIR/preseed/$SEEDFILENAME

	# Replace the boot menu
	rm -f $FINALDIR/isolinux/isolinux.cfg
	cp $ISOLINUXCFG $FINALDIR/isolinux/isolinux.cfg
	
	# Add the custom script for autopartitionning
	mkdir -p $FINALDIR/pool/main/a/autoparted
	cp $AUTOPARTEDUDEB $FINALDIR/pool/main/a/autoparted/

	# Remove all partman packages.
	rm -rf $FINALDIR/pool/main/p/partman-*

	build
}

while [ "$1" != "" ]; do
    case $1 in
	-h | --help )		usage
				exit
				;;
	-f | --full )
				full
				exit
				;;
	-r | --rebuild-only )
				build
				exit
				;;
	* )			usage
				exit 1
    esac
    shift
done
usage
exit 1
